﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Linq;
using System.ServiceProcess;
using System.Text;
using System.IO;
using System.Threading;
using System.Xml;
using System.Xml.Serialization;
using System.Configuration;
using Microsoft.EnterpriseManagement;
using Microsoft.EnterpriseManagement.Common;
using Microsoft.EnterpriseManagement.Monitoring.Security;
using Microsoft.EnterpriseManagement.Administration;
using Microsoft.EnterpriseManagement.ConnectorFramework;
using Microsoft.EnterpriseManagement.Configuration;
using Microsoft.EnterpriseManagement.Monitoring;
using System.Collections.ObjectModel;
using Microsoft.Win32;

namespace AlertUpdaterService
{
    public partial class Service1 : ServiceBase
    {
        public Service1()
        {
            InitializeComponent();
        }

        private Thread tMainThread;
        private bool bStopped = false;
        private bool bProcessing = false; //flag indicating that currently processing alerts or currently loading a file that changed.
        /// <summary>
        /// Configuration file settings from file
        /// </summary>
        private string strRootManagementServer = "";
        private string strPollingIntervalInSeconds = "";
        private string strConfigurationFilePath = "";
        private string strExcludedResolutionStates = "";


        /// <summary>
        /// Configuration of alerts to modify from connectorconfig.xml
        /// </summary>
        private System.Collections.Specialized.StringCollection scPropertiesModify = new System.Collections.Specialized.StringCollection();
        private System.Collections.Specialized.StringCollection scId = new System.Collections.Specialized.StringCollection();
        private System.Collections.Specialized.StringCollection scType = new System.Collections.Specialized.StringCollection();
        private System.Collections.Specialized.StringCollection scUserName = new System.Collections.Specialized.StringCollection();
        private System.Collections.Specialized.StringCollection scParamReplace = new System.Collections.Specialized.StringCollection();
        private System.Collections.Specialized.StringCollection scFailOverMS = new System.Collections.Specialized.StringCollection();
        private string strGlobalResolutionState = "251"; //default to 251



        protected override void OnStart(string[] args)
        {
            bStopped = false;
            tMainThread = new Thread(ConnectorMain);
            tMainThread.Start();
        }

        protected override void OnStop()
        {
            try
            {
                int iTimer = 3600; //wait 15 minutes Maximum should never take that long...
                bStopped = true;
                int iDelay = 0;
                while (tMainThread.IsAlive)
                {
                    Thread.Sleep(250);
                    iDelay++;
                    if (iDelay > iTimer)
                    {
                        eventLogAlertUpdater.WriteEntry("Alert Updater Service Failed to Stop", EventLogEntryType.Error, 13);
                        return; //exit 
                    }
                }
                eventLogAlertUpdater.WriteEntry("Alert Updater Service Stopped Successfully", EventLogEntryType.Information, 10);

            }
            catch (Exception eStop)
            {
                eventLogAlertUpdater.WriteEntry("Alert Updater Service Failed to Stop: " + eStop.Message.ToString(), EventLogEntryType.Error, 13);
            }
        }

        public bool LoadServiceConfiguration()
        {

            bool bReturn = true;
            try
            {
                scFailOverMS.Clear();
                string strTemp = System.Configuration.ConfigurationManager.AppSettings.Get("FailoverMS");
                string[] stemp = strTemp.Split(';');
                foreach (string s in stemp)
                {
                    scFailOverMS.Add(s);
                }
            }
            catch
            {
                //not required so no error
            }
            try
            {
                strRootManagementServer = System.Configuration.ConfigurationManager.AppSettings.Get("RootManagementServerName");
                strPollingIntervalInSeconds = System.Configuration.ConfigurationManager.AppSettings.Get("PollingIntervalInSeconds");
                strConfigurationFilePath = System.Configuration.ConfigurationManager.AppSettings.Get("ConfigurationFilePath");
                strExcludedResolutionStates = System.Configuration.ConfigurationManager.AppSettings.Get("ExcludedResolutionStates");
                if (strRootManagementServer.Length == 0)
                {
                    //log error
                    eventLogAlertUpdater.WriteEntry("RootManagementServer Configuration invalid - Check service configuration file for errors", EventLogEntryType.Error, 21);
                    bReturn = false;
                }
                if (strPollingIntervalInSeconds.Length == 0)
                {
                    //log error

                    eventLogAlertUpdater.WriteEntry("PollingIntervalInSeconds Configuration invalid - Check service configuration file for errors", EventLogEntryType.Error, 22);
                    bReturn = false;
                    //check integer validate
                    int iTemp;

                    if (!int.TryParse(strPollingIntervalInSeconds, out iTemp))
                    {
                        eventLogAlertUpdater.WriteEntry("PollingIntervalInSeconds Configuration not numeric - Check service configuration file for errors", EventLogEntryType.Error, 23);
                        bReturn = false;
                    }
                }
                if (strConfigurationFilePath.Length == 0)
                {
                    //log error
                    eventLogAlertUpdater.WriteEntry("ConfigurationFilePath Configuration invalid - Check service configuration file for errors", EventLogEntryType.Error, 24);
                    bReturn = false;
                }
                if (strExcludedResolutionStates.Length == 0)
                {
                    //log error
                    eventLogAlertUpdater.WriteEntry("ExcludedResolutionStates Configuration invalid - Check service configuration file for errors", EventLogEntryType.Error, 25);
                    bReturn = false;
                    int iTemp;
                    //check integer validate
                    if (!int.TryParse(strExcludedResolutionStates, out iTemp))
                    {
                        eventLogAlertUpdater.WriteEntry("ExcludedResolutionStates Configuration not numeric - Check service configuration file for errors", EventLogEntryType.Error, 26);
                        bReturn = false;
                    }

                }

                //Setup file watcher on strConfigurationFilePath
                string sPath = "";
                string sFile = "";
                try
                {
                    sPath = strConfigurationFilePath.Substring(0, strConfigurationFilePath.LastIndexOf("\\"));
                    sFile = strConfigurationFilePath.Substring(strConfigurationFilePath.LastIndexOf("\\") + 1);
                    fileSystemWatcherConfigFile.Path = sPath;
                    fileSystemWatcherConfigFile.Filter = sFile;
                    fileSystemWatcherConfigFile.EnableRaisingEvents = true;
                    eventLogAlertUpdater.WriteEntry("file path: " + sPath + ". File: " + sFile , EventLogEntryType.SuccessAudit, 33);

                }
                catch (Exception eCodeFile)
                {
                    // log could not setup config file watcher will have to restart service to load if file changes.
                    eventLogAlertUpdater.WriteEntry("Could not configure File Watcher, so will have to restart service if you change configuration file path: " + sPath + ". File: " + sFile + ": " + eCodeFile.InnerException.ToString(), EventLogEntryType.Error, 33);

                }
            }
            catch (Exception eCode)
            {
                // log error eventlog
                eventLogAlertUpdater.WriteEntry("LoadServiceConfiguration() Catch All Failure: " + eCode.InnerException.ToString(), EventLogEntryType.Error, 29);
                return false;
            }
            if (bReturn)
            {
                eventLogAlertUpdater.WriteEntry("Successfully Loaded Service Configuration", EventLogEntryType.Information, 20);
            }
            return bReturn;
        }

        /// <summary>
        /// LoadAlertPropertiesToModify:
        /// NOTE: if ever add a new replacement parameter you must add it to scParamReplace or it will never replace due to not finding in lookup on alerts
        /// </summary>
        /// <returns></returns>
        private bool LoadAlertPropertiesToModify()
        {

            try
            {
                scId.Clear();
                scType.Clear();
                scPropertiesModify.Clear();
                scUserName.Clear();
            }
            catch { }
            try
            {
                bool bReturn = true;
                bool bId;
                bool bUserName;
                bool bType;
                // check file exists
                if (!File.Exists(strConfigurationFilePath))
                {
                    // log error and exit. 
                    eventLogAlertUpdater.WriteEntry("LoadAlertPropertiesToModify() Configuration File NOT found: " + strConfigurationFilePath, EventLogEntryType.Error, 35);
                    return false;
                }
                XmlDocument xmlDoc = new XmlDocument();
                XmlTextReader xmlText = new XmlTextReader(strConfigurationFilePath);
                xmlDoc.Load(xmlText);

                XmlNodeList xNodelst = xmlDoc.DocumentElement.SelectNodes("//ConnectorConfig");//reading node so that we can traverse thorugh the XML
                foreach (XmlNode xNode in xNodelst)//traversing XML 
                {
                    foreach (XmlAttribute xa in xNode.Attributes)
                    {
                        if (xa.LocalName == "GlobalResolutionState")
                            strGlobalResolutionState = xa.Value; //got global resolution state
                    }
                }

                XmlNodeList xNodelstSources = xmlDoc.DocumentElement.SelectNodes("//ConnectorConfig/AlertSources/AlertSource");//reading node so that we can traverse thorugh the XML

                foreach (XmlNode xNode in xNodelstSources)//traversing XML 
                {
                    //...code here
                    bId = false; //just in case someone has multiple of these and don't want to add more than one
                    bType = false;
                    bUserName = false;
                    foreach (XmlAttribute xa in xNode.Attributes)
                    {
                        //Id Type UserName
                        if (xa.LocalName == "Id" && !bId)
                        {
                            scId.Add(xa.Value); //got id
                            bId = true;
                        }
                        if (xa.LocalName == "Type" && !bType)
                        {
                            scType.Add(xa.Value); //got type
                            bType = true;
                        }
                        if (xa.LocalName == "UserName" && !bUserName)
                        {
                            scUserName.Add(xa.Value); //got username
                            bUserName = true;
                        }
                    }
                    //Now traverse this node for Propertiestomodify/properties
                    foreach (XmlNode xNodePropMod in xNode.ChildNodes)
                    {
                        string strProp = ""; //create new string to build with properties
                        foreach (XmlNode xNodeProperty in xNodePropMod.ChildNodes)
                        {
                            //this should be all properties if count greater than one we are good.
                            foreach (XmlAttribute xaP in xNodeProperty.Attributes)
                            {
                                if (xaP.Value.Length < 1)
                                    strProp += " =";
                                else
                                    strProp += xaP.Value + "=";
                            }
                            strProp = strProp.TrimEnd('=');
                            strProp += ";";

                        }
                        //populate our string collection scProperties
                        strProp = strProp.TrimEnd(';');
                        scPropertiesModify.Add(strProp);
                    }

                }



                //cleanup
                xmlText.Close();
                eventLogAlertUpdater.WriteEntry("Successfully Loaded Properties to Modify Configuration", EventLogEntryType.Information, 30);
                // Debugging purposes: Dump to text file for debug purposes remove when final
                //try
                //{
                //    StreamWriter sw = new StreamWriter(".\\AUCPropertyLog.txt",false);

                //    int i = 0;
                //    foreach (string s in scId)
                //    {
                //        sw.WriteLine("id: " + s);
                //        sw.WriteLine("Type: " + scType[i]);
                //        sw.WriteLine("UserName: " + scUserName[i]);
                //        sw.WriteLine("Properties: " + scPropertiesModify[i]);
                //        i++;
                //        sw.WriteLine();
                //        sw.WriteLine();
                //    }
                //    sw.Close();
                //    sw.Dispose();
                //}
                //catch (Exception eFile)
                //{
                //    // log event log failed write AUCPropertyLog.txt
                //    //still return true as we loaded config ok, however may have issues debugging if log needed.
                //    eventLogAlertUpdater.WriteEntry("Failed to write log file for Properties to Modify Configuration: " + eFile.InnerException.ToString(), EventLogEntryType.Warning, 38);
                //}
                return bReturn;
            }
            catch (Exception eCode)
            {
                eventLogAlertUpdater.WriteEntry("LoadAlertPropertiesToModify() Catch all failure: " + eCode.InnerException.ToString(), EventLogEntryType.Error, 39);
                return false; //log event and return false
            }

        }

        /// <summary>
        /// This is the main thread for Alert Updater Connector Service Example
        /// </summary>
        public void ConnectorMain()
        {

            bStopped = false;
            try
            {
                // Call Load Configuration new function load frsom XML to populate everything.
                if (!LoadServiceConfiguration())
                {
                    // log error event log
                    eventLogAlertUpdater.WriteEntry("Failed in LoadServiceConfiguration(). Check event log for additional details. Exiting service", EventLogEntryType.Error, 51);
                    bStopped = true;
                    return; //stop service
                }

                if (!LoadAlertPropertiesToModify())
                {
                    // log error event log
                    eventLogAlertUpdater.WriteEntry("Failed in LoadAlertPropertiesToModify(). Check event log for additional details. Exiting service", EventLogEntryType.Error, 52);
                    bStopped = true;
                    return;
                }

            }
            catch (Exception eConfig)
            {
                // log error for config file log in main thread.
                eventLogAlertUpdater.WriteEntry("Failed Loading configuration files. Check event log for additional details. Exiting service: " + eConfig.InnerException.ToString(), EventLogEntryType.Error, 53);
                bStopped = true;
                return;
            }

            //Added failover functionality.
            DateTime dtLastConnectTime = DateTime.Now; //initialize last Connect time.
            string strSaveOriginalRMS = strRootManagementServer;
            int iConnect = 0; //Number of recent retry connection attempts
            bool bExhaustedRetries = false;
            while (!bStopped)
            {
                System.Threading.Thread.Sleep(10000);
                // if we remain connected for greater than 30 minutes to a given MS and fail again reset back to original MS specified for reestablishing connection to preferred MS
                if (dtLastConnectTime > DateTime.Now.AddMinutes(30))
                {
                    // If we stayed connected for over 30 minutes to same MS then we set back to original MS and reset iConnect to 0
                    strRootManagementServer = strSaveOriginalRMS;
                    iConnect = 0;
                    bExhaustedRetries = false;
                }
                dtLastConnectTime = DateTime.Now; //update last connection attempt time.
                if (!CallMain())
                {
                    //failed so use failoverMS list if available.
                    if (scFailOverMS.Count > 0)
                    {
                        if (iConnect > scFailOverMS.Count)
                        {
                            strRootManagementServer = strSaveOriginalRMS; //try origal MS one time
                            if (bExhaustedRetries)
                            {
                                //We are done. Log failed connecting all MS's and exit stop service.
                                eventLogAlertUpdater.WriteEntry("Failed last attempt to reconnect MS list to Management Group. Fatal error check health of RMS/MS and restart service.", EventLogEntryType.Error, 5011);
                                bStopped = true;
                            }
                            bExhaustedRetries = true;
                        }
                        else
                        {
                            //set strRootManagementServer to MS Failover to try connect.
                            strRootManagementServer = scFailOverMS[iConnect];
                            eventLogAlertUpdater.WriteEntry("Changing connection to failover MS: " + strRootManagementServer + ". Will attempt to connect to new MS.", EventLogEntryType.Information, 5002);
                        }
                    }
                    else
                    {
                        if (bExhaustedRetries)
                        {
                            //todo log event exhausted retries and exit fatal error restart service and check Health of RMS/MS
                            eventLogAlertUpdater.WriteEntry("Failed last attempt to reconnect to Management Group. Fatal error check health of RMS/MS and restart service.", EventLogEntryType.Error, 5010);
                            bStopped = true;
                        }
                        bExhaustedRetries = true;
                    }
                    iConnect++; //increment connection counter
                    dtLastConnectTime = DateTime.Now;
                }
                else
                {

                }
                //
            }
        }
 
        private bool CallMain()
        {
            try
            {
                //Now we connect to the specified Management group from the configuration file
                ManagementGroup mg = new ManagementGroup(strRootManagementServer);
                eventLogAlertUpdater.WriteEntry("Successfully connected to SDK service on: " + strRootManagementServer, EventLogEntryType.Information, 50);
                ConnectorFrameworkAdministration cfAdmin = mg.GetConnectorFrameworkAdministration();
                Guid connectorGuid = new Guid("{eac9ef4a-6e51-4d23-bcc0-153cc733d659}");
                MonitoringConnector connector;
                try
                {
                    try
                    {
                        connector = cfAdmin.GetMonitoringConnector(connectorGuid);
                        eventLogAlertUpdater.WriteEntry("Successfully established connector connection", EventLogEntryType.Information, 55);
                    }
                    catch
                    {
                        connector = null;
                        eventLogAlertUpdater.WriteEntry("Could not locate Monitoring Connector. Will attempt to create connector.", EventLogEntryType.Information, 100);
                    }
                    try
                    {
                        if (connector == null)
                        {
                            ConnectorInfo info = new ConnectorInfo();
                            info.Description = "Alert Updater Connector allows for updating of fields from the alert values";
                            info.DisplayName = "Alert Updater Connector";
                            info.Name = "Alert Updater connector";
                            connector = cfAdmin.Setup(info, connectorGuid);
                            connector.Initialize();
                            eventLogAlertUpdater.WriteEntry("Successfully created Alert Updater Connector.", EventLogEntryType.Information, 119);
                        }
                    }
                    catch (Exception e9)
                    {
                        eventLogAlertUpdater.WriteEntry("Failed to create connector Exiting: " + e9.Message.ToString(), EventLogEntryType.Error, 101);
                        bStopped = true;
                        return false; //stop service.
                    }
                    try
                    {
                        connector = cfAdmin.GetMonitoringConnector(connectorGuid);
                    }
                    catch (Exception eConn)
                    {
                        eventLogAlertUpdater.WriteEntry("Failed to get monitoring connector Exiting: " + eConn.Message.ToString(), EventLogEntryType.Error, 102);
                        bStopped = true;
                        return false; //stop service.
                    }

                    int iWriteHeartBeat = 0; //use this to write event occasionally to let users know service is still actively processing alerts
                    int iPollingSeconds = 10; //default to 10 seconds
                    try
                    {
                        iPollingSeconds = int.Parse(strPollingIntervalInSeconds);
                    }
                    catch
                    {
                        iPollingSeconds = 10;
                    }
                    int iChangeCheck = 0;
                    iPollingSeconds = iPollingSeconds * 100; //convert to milliseconds leave gap for 10 iterations for config file change check
                    while (!bStopped)
                    {
                        bProcessing = false;
                        iChangeCheck = 0;

                        while (iChangeCheck < 10)
                        {
                            System.Threading.Thread.Sleep(iPollingSeconds); //polling seconds interval
                            iChangeCheck++;
                            while (bProcessing)
                            {
                                System.Threading.Thread.Sleep(100); //wait for file reload to occur then continue.
                            }
                        }
                        bProcessing = true; //set flag that we are processing alerts and do not load config file should it change and we are still processing

                        //Can use for telling if service is still running think Yes and create MP for the Connector.
                        //> 10 means log about every 2 minutes, adjust to your needs or remove if do not want heartbeat in service
                        iWriteHeartBeat++;
                        if (iWriteHeartBeat > 10) //change this to however often you want to log a informational event to event log
                        {
                            iWriteHeartBeat = 0;
                            eventLogAlertUpdater.WriteEntry("Processing Alerts Now. Connected to: " + strRootManagementServer, EventLogEntryType.Information, 700);
                        }
                        // populate webconsoleURL
                        //: "http://SCOMRMS:51908/default.aspx?DisplayMode=Pivot&AlertID=%7b917095f2-a3ab-4fb9-b3cf-bbda89d550d2%7d"
                        string strWebConsoleURL = "not found";
                        try
                        {
                            strWebConsoleURL = mg.GetAdministration().Settings.GetDefaultValue(Settings.NotificationServer.WebAddresses.WebConsole);
                        }
                        catch
                        { }
                        ReadOnlyCollection<ConnectorMonitoringAlert> aProcessAlerts;
                        aProcessAlerts = connector.GetMonitoringAlerts();
                        //acknoledge alerts
                        if (aProcessAlerts.Count > 0)
                        {
                            connector.AcknowledgeMonitoringAlerts(aProcessAlerts);
                        }
                        foreach (ConnectorMonitoringAlert alert in aProcessAlerts)
                        {
                            //alert resolutions we want to ignore
                            try
                            {
                                if (alert.ResolutionState == int.Parse(strExcludedResolutionStates) || alert.ResolutionState == int.Parse(strGlobalResolutionState))
                                {
                                    continue; //ignore these resolutions states
                                }
                            }
                            catch { }
                            int iIndex = scId.IndexOf(alert.MonitoringRuleId.ToString());
                            if (iIndex >= 0) //found a match
                            {
                                try
                                {
                                    string strServerName = "";
                                    try
                                    {
                                        strServerName = alert.NetbiosComputerName;
                                    }
                                    catch
                                    {  }
                                    //fix for when HealthService Watcher is target and NetBiosComputerName fails.
                                    try
                                    {
                                        if (strServerName.Length < 1)
                                            strServerName = alert.MonitoringObjectDisplayName;
                                    }
                                    catch
                                    {
                                        //Catch due to length check
                                        try
                                        {
                                            strServerName = alert.MonitoringObjectDisplayName; //just set to Monitoring object display name for HS Watcher failures
                                        }
                                        catch { }
                                    }
                                    //Set to processed state first, then if specified value different it will set it below before Update occurs
                                    alert.ResolutionState = byte.Parse(strGlobalResolutionState);
                                    //"ResolutionState=252= ;CustomField1=$ServerName$= "
                                    // field=value=groupid if exists;field=value=groupid if exists;.......
                                    string[] sAlertConfig = scPropertiesModify[iIndex].Split(';');
                                   
                                    foreach (string sItems in sAlertConfig)
                                    {
                                        string[] sSplit = sItems.Split('=');
                                        //Now we have sSplit 0 = Field to modify, 1 = new value to set, 2 = Group Guid ID to apply this to
                                        if (sSplit[2].Length > 5) //we have a guid and need to apply only IF this server is in that group specified
                                        {
                                            //Get group and determine if this server is in that group If not continue to next item in foreach
                                            MonitoringClassCriteria classCriteria =
                                                new MonitoringClassCriteria("Id = '" + sSplit[2] + "'");

                                            // There should only be one item in the monitoringClasses collection.
                                            ReadOnlyCollection<MonitoringClass> monitoringClasses =
                                                mg.GetMonitoringClasses(classCriteria);

                                            // Get all instances of the specified GroupFilterId in the management group. If this server from this alert is not in that group then do not update alert.
                                            ReadOnlyCollection<MonitoringObject> monitoringObjects =
                                                mg.GetMonitoringObjects(monitoringClasses[0]);
                                            bool bMatch = false;
                                            foreach (MonitoringObject mon in monitoringObjects)
                                            {
                                                foreach (MonitoringObject mon2 in mon.GetRelatedMonitoringObjects())
                                                {
                                                    //these are the FQDN's displayname that we need to match on
                                                    if (strServerName == mon2.DisplayName)
                                                    {
                                                        bMatch = true;
                                                        continue;
                                                    }
                                                }
                                            }

                                            if (!bMatch)
                                            {
                                                continue;//skip this one as the server is not in the Group to apply this change
                                            }
                                        }
                                        //Now get Value to set
                                        string strValueSet = sSplit[1];
                                        if (strValueSet[0] == '$' && strValueSet[strValueSet.Length - 1] == '$')
                                        {
                                            //We know this is a replacement parameter, so get the replacement string to set field with.
                                            //Commment out Case statements if you do not want to replace those params, as desired.
                                            switch (strValueSet)
                                            {
                                                case "$ServerName$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$ServerName$", strServerName);
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$ServerName$", "Null value");
                                                    }

                                                    break;
                                                case "$WebConsoleURL$":
                                                    try
                                                    {
                                                        //: "http://SCOMRMS:51908/default.aspx?DisplayMode=Pivot&AlertID=%7b917095f2-a3ab-4fb9-b3cf-bbda89d550d2%7d"

                                                        strValueSet = strValueSet.Replace("$WebConsoleURL$", strWebConsoleURL + "?DisplayMode=Pivot&AlertID=%7b" + alert.Id.ToString() + "%7d");
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$WebConsoleURL$", "Null value");
                                                    }

                                                    break;
                                                case "$Category$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$Category$", alert.Category.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$Category$", "Null value");
                                                    }

                                                    break;
                                                case "$ConnectorId$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$ConnectorId$", alert.ConnectorId.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$ConnectorId$", "Null value");
                                                    }
                                                    break;
                                                case "$ConnectorStatus$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$ConnectorStatus$", alert.ConnectorStatus.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$ConnectorStatus$", "Null value");
                                                    }
                                                    break;
                                                case "$Context$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$Context$", alert.Context.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$Context$", "Null value");
                                                    }
                                                    break;
                                                case "$CustomField1$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$CustomField1$", alert.CustomField1.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$CustomField1$", "Null value");
                                                    }
                                                    break;
                                                case "$CustomField10$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$CustomField10$", alert.CustomField10.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$CustomField10$", "Null value");
                                                    }
                                                    break;
                                                case "$CustomField2$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$CustomField2$", alert.CustomField2.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$CustomField2$", "Null value");
                                                    }
                                                    break;
                                                case "$CustomField3$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$CustomField3$", alert.CustomField3.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$CustomField3$", "Null value");
                                                    }
                                                    break;
                                                case "$CustomField4$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$CustomField4$", alert.CustomField4.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$CustomField4$", "Null value");
                                                    }
                                                    break;
                                                case "$CustomField5$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$CustomField5$", alert.CustomField5.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$CustomField5$", "Null value");
                                                    }
                                                    break;
                                                case "$CustomField6$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$CustomField6$", alert.CustomField6.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$CustomField6$", "Null value");
                                                    }
                                                    break;
                                                case "$CustomField7$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$CustomField7$", alert.CustomField7.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$CustomField7$", "Null value");
                                                    }
                                                    break;
                                                case "$CustomField8$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$CustomField8$", alert.CustomField8.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$CustomField8$", "Null value");
                                                    }
                                                    break;
                                                case "$CustomField9$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$CustomField9$", alert.CustomField9.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$CustomField9$", "Null value");
                                                    }
                                                    break;
                                                case "$Description$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$Description$", alert.Description.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$Description$", "Null value");
                                                    }
                                                    break;
                                                case "$Id$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$Id$", alert.Id.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$Id$", "Null value");
                                                    }
                                                    break;
                                                case "$IsMonitorAlert$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$IsMonitorAlert$", alert.IsMonitorAlert.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$IsMonitorAlert$", "Null value");
                                                    }
                                                    break;
                                                case "$LastModified$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$LastModified$", alert.LastModified.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$LastModified$", "Null value");
                                                    }
                                                    break;
                                                case "$LastModifiedBy$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$LastModifiedBy$", alert.LastModifiedBy.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$LastModifiedBy$", "Null value");
                                                    }
                                                    break;
                                                case "$LastModifiedByNonConnector$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$LastModifiedByNonConnector$", alert.LastModifiedByNonConnector.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$LastModifiedByNonConnector$", "Null value");
                                                    }
                                                    break;
                                                case "$MaintenanceModeLastModified$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$MaintenanceModeLastModified$", alert.MaintenanceModeLastModified.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$MaintenanceModeLastModified$", "Null value");
                                                    }
                                                    break;
                                                case "$ManagementGroup$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$ManagementGroup$", alert.ManagementGroup.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$ManagementGroup$", "Null value");
                                                    }
                                                    break;
                                                case "$ManagementGroupId$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$ManagementGroupId$", alert.ManagementGroupId.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$ManagementGroupId$", "Null value");
                                                    }
                                                    break;
                                                case "$ManagementGroupName$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$ManagementGroupName$", alert.ManagementGroupName.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$ManagementGroupName$", "Null value");
                                                    }
                                                    break;
                                                case "$MonitoringClassId$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$MonitoringClassId$", alert.MonitoringClassId.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$MonitoringClassId$", "Null value");
                                                    }
                                                    break;
                                                case "$MonitoringObjectDisplayName$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$MonitoringObjectDisplayName$", alert.MonitoringObjectDisplayName.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$MonitoringObjectDisplayName$", "Null value");
                                                    }
                                                    break;
                                                case "$MonitoringObjectFullName$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$MonitoringObjectFullName$", alert.MonitoringObjectFullName.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$MonitoringObjectFullName$", "Null value");
                                                    }
                                                    break;
                                                case "$MonitoringObjectHealthState$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$MonitoringObjectHealthState$", alert.MonitoringObjectHealthState.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$MonitoringObjectHealthState$", "Null value");
                                                    }
                                                    break;
                                                case "$MonitoringObjectId$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$MonitoringObjectId$", alert.MonitoringObjectId.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$MonitoringObjectId$", "Null value");
                                                    }
                                                    break;
                                                case "$MonitoringObjectInMaintenanceMode$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$MonitoringObjectInMaintenanceMode$", alert.MonitoringObjectInMaintenanceMode.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$MonitoringObjectInMaintenanceMode$", "Null value");
                                                    }
                                                    break;
                                                case "$MonitoringObjectName$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$MonitoringObjectName$", alert.MonitoringObjectName.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$MonitoringObjectName$", "Null value");
                                                    }
                                                    break;
                                                case "$MonitoringObjectPath$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$MonitoringObjectPath$", alert.MonitoringObjectPath.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$MonitoringObjectPath$", "Null value");
                                                    }
                                                    break;
                                                case "$MonitoringRuleId$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$MonitoringRuleId$", alert.MonitoringRuleId.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$MonitoringRuleId$", "Null value");
                                                    }
                                                    break;
                                                case "$Name$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$Name$", alert.Name.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$Name$", "Null value");
                                                    }
                                                    break;
                                                case "$NetbiosComputerName$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$NetbiosComputerName$", alert.NetbiosComputerName.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$NetbiosComputerName$", "Null value");
                                                    }
                                                    break;
                                                case "$NetbiosDomainName$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$NetbiosDomainName$", alert.NetbiosDomainName.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$NetbiosDomainName$", "Null value");
                                                    }
                                                    break;
                                                case "$Owner$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$Owner$", alert.Owner.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$Owner$", "Null value");
                                                    }
                                                    break;
                                                case "$Parameters$":
                                                    try
                                                    {
                                                        //This is read only collection most likely have to change how get or remove...
                                                        //example of one you may want to remove or modify to suit your needs.
                                                        strValueSet = strValueSet.Replace("$Parameters$", alert.Parameters.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$Parameters$", "Null value");
                                                    }
                                                    break;
                                                case "$PrincipalName$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$PrincipalName$", alert.PrincipalName.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$PrincipalName$", "Null value");
                                                    }
                                                    break;
                                                case "$Priority$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$Priority$", alert.Priority.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$Priority$", "Null value");
                                                    }
                                                    break;
                                                case "$ProblemId$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$ProblemId$", alert.ProblemId.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$ProblemId$", "Null value");
                                                    }
                                                    break;
                                                case "$RepeatCount$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$RepeatCount$", alert.RepeatCount.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$RepeatCount$", "Null value");
                                                    }
                                                    break;
                                                case "$ResolutionState$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$ResolutionState$", alert.ResolutionState.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$ResolutionState$", "Null value");
                                                    }
                                                    break;
                                                case "$ResolvedBy$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$ResolvedBy$", alert.ResolvedBy.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$ResolvedBy$", "Null value");
                                                    }
                                                    break;
                                                case "$Severity$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$Severity$", alert.Severity.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$Severity$", "Null value");
                                                    }
                                                    break;
                                                case "$SiteName$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$SiteName$", alert.SiteName.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$SiteName$", "Null value");
                                                    }
                                                    break;
                                                case "$StateLastModified$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$StateLastModified$", alert.StateLastModified.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$StateLastModified$", "Null value");
                                                    }
                                                    break;
                                                case "$TicketId$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$TicketId$", alert.TicketId.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$TicketId$", "Null value");
                                                    }
                                                    break;
                                                case "$TimeAdded$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$TimeAdded$", alert.TimeAdded.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$TimeAdded$", "Null value");
                                                    }
                                                    break;
                                                case "$TimeRaised$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$TimeRaised$", alert.TimeRaised.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$TimeRaised$", "Null value");
                                                    }
                                                    break;
                                                case "$TimeResolutionStateLastModified$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$TimeResolutionStateLastModified$", alert.TimeResolutionStateLastModified.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$TimeResolutionStateLastModified$", "Null value");
                                                    }
                                                    break;
                                                case "$TimeResolved$":
                                                    try
                                                    {
                                                        strValueSet = strValueSet.Replace("$TimeResolved$", alert.TimeResolved.ToString());
                                                    }
                                                    catch
                                                    {
                                                        strValueSet = strValueSet.Replace("$TimeResolved$", "Null value");
                                                    }
                                                    break;
                                                default:
                                                    break;

                                            }
                                        }
                                        //Now set the values from strValueSet to correct field.
                                        //Values you can set are: CustomField1 - 10, Owner, ResolutionState, TicketId
                                        // strValueSet need to limit to 255 characters as that is the max size limit on setting these fields.
                                        //eventLogAlertUpdater.WriteEntry("before length check " + strValueSet, EventLogEntryType.Information, 3);
                                        if (strValueSet.Length > 255)
                                        {
                                            try
                                            {
                                                strValueSet = strValueSet.Remove(254);
                                            }
                                            catch { }
                                        }
                                        switch (sSplit[0])
                                        {
                                            case "ResolutionState":
                                                try
                                                {
                                                    alert.ResolutionState = byte.Parse(strValueSet.Trim());
                                                }
                                                catch { }
                                                break;
                                            case "CustomField1":
                                                alert.CustomField1 = strValueSet;
                                                break;
                                            case "CustomField2":
                                                alert.CustomField2 = strValueSet;
                                                break;
                                            case "CustomField3":
                                                alert.CustomField3 = strValueSet;
                                                break;
                                            case "CustomField4":
                                                alert.CustomField4 = strValueSet;
                                                break;
                                            case "CustomField5":
                                                alert.CustomField5 = strValueSet;
                                                break;
                                            case "CustomField6":
                                                alert.CustomField6 = strValueSet;
                                                break;
                                            case "CustomField7":
                                                alert.CustomField7 = strValueSet;
                                                break;
                                            case "CustomField8":
                                                alert.CustomField8 = strValueSet;
                                                break;
                                            case "CustomField9":
                                                alert.CustomField9 = strValueSet;
                                                break;
                                            case "CustomField10":
                                                alert.CustomField10 = strValueSet;
                                                break;
                                            case "Owner":
                                                try
                                                {
                                                    alert.Owner = strValueSet;
                                                }
                                                catch
                                                {
                                                    //ignore
                                                }
                                                break;
                                            case "TicketId":
                                                try
                                                {
                                                    alert.TicketId = strValueSet;
                                                }
                                                catch
                                                {
                                                    //ignore
                                                }
                                                break;
                                            default:
                                                break;
                                        }
                                    }
                                    //Now that we have set all the selected fields update the alert
                                    try
                                    {
                                        eventLogAlertUpdater.WriteEntry("Updating alert with changes now: " + alert.Name, EventLogEntryType.Information, 799);
                                        alert.ConnectorId = null;

                                        try
                                        {
                                            alert.Update("Alert Updater Service - Processed Alert with changes");

                                        }
                                        catch
                                        {
                                            //this is not an error on SCOM 2012... Seems to update alert fine, but get exception.
                                            //Let's check for less than 2012 version and then write bad event otherwise we ignore.
                                            eventLogAlertUpdater.WriteEntry("Update alert catch, however if SCOM 2012 ignore this, as appears ok. try build with latest SDK binaries", EventLogEntryType.Warning, 555);
                                        }
                                        //eventLogAlertUpdater.WriteEntry("after Update alert", EventLogEntryType.Information, 5);
                                    }
                                    catch (Exception ePropReplace)
                                    {
                                        //ignore cases where fail due to something else changing the alert before we get a chance to update 
                                        //believe this is the behaviour of AUC
                                        eventLogAlertUpdater.WriteEntry("Failed replacing Properties updating alert(ignore most cases): " + scPropertiesModify[iIndex] + " : " + ePropReplace.InnerException.ToString(), EventLogEntryType.Error, 711);
                                    }


                                }
                                catch (Exception eP)
                                {
                                    //log event or ignore if parsing fails
                                    eventLogAlertUpdater.WriteEntry("Failed parsing Properties: " + scPropertiesModify[iIndex] + ": " + eP.Message.ToString(), EventLogEntryType.Error, 710);
                                }

                            }
                            else
                            {
                                //Just set to strGlobalResolutionState and update alert as we have processed it.
                                alert.ResolutionState = byte.Parse(strGlobalResolutionState);
                                alert.ConnectorId = null;
                                try
                                {
                                    alert.Update("Alert Updater Service - Processed Alert with no changes");
                                }
                                catch
                                {
                                    //ignore cases where fail due to something else changing the alert before we get a chance to update
                                }

                            }

                        }
                        //Acknowledge alerts that we processed, so connector does not continually process these.
                        //todo, should we move this up to top? or leave here for other connector to pick up after update? test out.
                        if (aProcessAlerts.Count > 0)
                        {
                            try
                            {
                                connector.AcknowledgeMonitoringAlerts(aProcessAlerts);
                            }
                            catch { }

                        }
                    }

                }
                catch (MonitoringException error)
                {
                    //This occurs if SDK service is stopped.
                    eventLogAlertUpdater.WriteEntry("Service failed. Please restart service: " + error.Message.ToString(), EventLogEntryType.Error, 7010);
                    return false; //return false so we will try to connect to another MS if available.
                }
            }
            catch (Exception eMain)
            {
                //todo log error
                eventLogAlertUpdater.WriteEntry("Service failed. Please restart service: " + eMain.Message.ToString(), EventLogEntryType.Error, 7012);
                return false; //return false so we will try to connect to another MS if available.
            }
            return true;

        }
        private void fileSystemWatcherConfigFile_Changed(object sender, FileSystemEventArgs e)
        {
            //when file changes load file again
            try
            {
                while (bProcessing)
                {
                    System.Threading.Thread.Sleep(100); //sleep 100 miliseconds before check again
                }
                bProcessing = true; //set so alerts will not process
                //log message detected change in config file reloading.
                eventLogAlertUpdater.WriteEntry("Detected Change in configuration file reloading.", EventLogEntryType.Information, 31);

                try
                {
                    LoadAlertPropertiesToModify(); //reload config file
                }
                catch
                {
                }

                bProcessing = false; //set back so continue processing alerts.
            }
            catch (Exception eCode)
            {
                //log error
                eventLogAlertUpdater.WriteEntry("Problem Reloading configuration file Please restart service: " + eCode.InnerException.ToString(), EventLogEntryType.Error, 32);

                bProcessing = false;
            }

        }


        //End
    }
}

